// Layer.cpp: implementation of the CLayer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Layer.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IMPLEMENT_SERIAL(CLayer, CObject, VERSIONABLE_SCHEMA | GEOIO_MODULE_VERSION_NUM)

CLayer::CLayer()
{
	m_strLayerName = _T("");
	m_clrLayer = RGB(255, 255, 255);
	m_nLayerID = 1;
	m_bLayerVisible = TRUE;
	m_bActiveLayer = FALSE;
	m_pEntityList = NULL;
	m_pEntityList = new CEntityList();

	// undo list pointer
	m_pUndoList = new CUndoList();
	m_nEntityNumber = 1;
}

CLayer::CLayer(CLayer& pLayer)
{
	int nEntitiyCount = pLayer.GetEntityCount();
	
	for( int ni=0; ni<nEntitiyCount; ni++ )
	{
		CPrimitive* pEntity = NULL,
				 *pCloneEntity = NULL;

		pEntity = (&pLayer)->GetEntityAtIndex(ni);
		pCloneEntity = pEntity->CreateClone();
		
		// add the cloned entity to new layer created //
		AddEntityToLayer(pCloneEntity);
	}
}

CLayer::~CLayer()
{
	// delete the entities in this layer //
	if( m_pEntityList != NULL )
	{
		delete m_pEntityList;
		m_pEntityList = NULL;
	}

	// delete all entities in the undo list //
	if( m_pUndoList != NULL )
	{
		delete m_pUndoList;
		m_pUndoList = NULL;
	}
}

void CLayer::Serialize(CArchive &ar)
{
	if (ar.IsStoring())
	{	// storing code
		ar << m_strLayerName;
		ar << m_clrLayer;
		ar << m_nLayerID;
		ar << m_bLayerVisible;
		ar << m_bActiveLayer;
	}
	else
	{	// loading code
		ar >> m_strLayerName;
		ar >> m_clrLayer;
		ar >> m_nLayerID;
		ar >> m_bLayerVisible;
		ar >> m_bActiveLayer;
	}
	m_pEntityList->Serialize(ar);
}


/**************************************************************************
	this function adds the entity to the current active layer
**************************************************************************/
void CLayer::AddEntityToLayer(CPrimitive *pEntity)
{
	pEntity->SetEntityColor(m_clrLayer);
	pEntity->SetEntityNumber(m_nEntityNumber);
	m_nEntityNumber++;
	m_pEntityList->AddEntity(pEntity);
}

/**************************************************************************
	this function draws all the entities in the layer
**************************************************************************/
void CLayer::DrawLayer(CDC *pDC)
{
	m_pEntityList->DrawEntityList(pDC);
}

/**************************************************************************
	this function changes the entities color in this layer
**************************************************************************/
void CLayer::ChangeEntitiesColorInThisLayer(COLORREF clr)
{
	m_pEntityList->ChangeEntitiesColorInList(clr);
}

/**************************************************************************
	this function erases the entity near the point passed as parmeter
**************************************************************************/
void CLayer::EraseEntityNearPoint(CPoint pnt)
{
	m_pEntityList->EraseEntityNearPoint(pnt);
}

/**************************************************************************
	this function gets the last entity in the entity list for undo 
	operation
**************************************************************************/
CPrimitive* CLayer::GetEntityForUndoOperation()
{
	CPrimitive *pEntity = m_pEntityList->GetLastEntity();
	
	return pEntity;
}

/**************************************************************************
	this function gets the last entity in the entity list for undo 
	operation
**************************************************************************/
CPrimitive* CLayer::GetEntityForRedoOperation()
{
	CPrimitive *pEntity = m_pUndoList->GetLastItem();
	
	return pEntity;
}

/**************************************************************************
	this function add an entity to undo list
**************************************************************************/
void CLayer::AddEntityToUndoList(CPrimitive *pPrimitive)
{
	m_pUndoList->AddToUndoList(pPrimitive);
}

/**************************************************************************************
	this function gets the number of entities in ths layer
**************************************************************************************/
int CLayer::GetEntityCount()
{
	return m_pEntityList->GetEntityCount();
}

/**************************************************************************************
	this function gets the number of entities in the undo list of
**************************************************************************************/
int CLayer::GetEntityCountInUndoList()
{
	return m_pUndoList->GetCount();
}

/**************************************************************************************
	this function deletes the entity from the layer when the entity number is passed
**************************************************************************************/
void CLayer::DeleteEntityFromLayer(int nEntityNumber)
{
	m_pEntityList->DeleteEntity(nEntityNumber);
}

/**************************************************************************************
	this function deletes the entity from the undo list when the 
	entity number is passed
**************************************************************************************/
void CLayer::DeleteEntityFromUndoList(int nEntityNumber)
{
	m_pUndoList->DeleteEntity(nEntityNumber);
}


/**************************************************************************************
 Function gets the entity at specified index from the list
**************************************************************************************/
CPrimitive* CLayer::GetEntityAtIndex(int nIndex)
{
	return m_pEntityList->GetEntityAtIndex(nIndex);
}

/**************************************************************************************
 Function gets the entity at specified index from the list
**************************************************************************************/
void CLayer::ResetColorOfAllEntities(COLORREF clr)
{
	m_pEntityList->ChangeEntitiesColorInList(clr);
}

/*---------------------------------------------------------------------------------
	Function to clone the layer. This function returns a new pointer to
	cloned layer.
---------------------------------------------------------------------------------*/
CLayer* CLayer::CreateClone()
{
	CLayer* pClone = new CLayer(*this);
	return pClone;
}
