/*
 *  By: Sunil Kumar
 *  GeoIO Module. Module to draw simple 2D geometry objects and serialization.
 *  This code is free and is distributed in the hope that it will be useful for simple learning and experimental purpose only WITHOUT any WARRANTY.
 *  The author shall hold no redistribution liability in case of any unethical usage/modification.
 *  Distributed under CPOL.
 */
// Layer.h: interface for the CLayer class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __LAYER_H__
#define __LAYER_H__

#include "stdafx.h"
#include "EntityList.h"
#include "UndoList.h"

class GEOIO_EXPORT CLayer : public CObject  
{
	DECLARE_SERIAL(CLayer)
public:
	CLayer();
	CLayer(CLayer& pLayer);
	virtual ~CLayer();

private:
	CString m_strLayerName;
	COLORREF m_clrLayer;
	int m_nLayerID;
	BOOL m_bLayerVisible;
	BOOL m_bActiveLayer;
	CEntityList *m_pEntityList;
	CUndoList *m_pUndoList;

public:
	CLayer* CreateClone();
	void ResetColorOfAllEntities(COLORREF clr);
	CPrimitive* GetEntityAtIndex(int nIndex);
	int m_nEntityNumber;
	void DeleteEntityFromLayer(int nEntityNumber);
	void DeleteEntityFromUndoList(int nEntityNumber);
	int GetEntityCountInUndoList();
	int GetEntityCount();
	void AddEntityToUndoList(CPrimitive *pPrimitive);
	CPrimitive* GetEntityForUndoOperation();
	CPrimitive* GetEntityForRedoOperation();
	void EraseEntityNearPoint(CPoint pnt);
	void ChangeEntitiesColorInThisLayer(COLORREF clr);
	void DrawLayer(CDC *pDC);
	void AddEntityToLayer(CPrimitive *pEntity);
	CString GetLayerName(){return m_strLayerName;}
	int GetLayerID(){return m_nLayerID;}
	COLORREF GetLayerColor(){return m_clrLayer;}
	int GetEntitiesCountInThisLayer(){ return (m_pEntityList->GetEntityCount()); }
	BOOL IsLayerVisible(){ return m_bLayerVisible;}
	BOOL IsActiveLayer(){ return m_bActiveLayer;}


	void SetLayerName(CString strName){m_strLayerName = strName;}
	void SetLayerID(int nID){m_nLayerID = nID;}
	void SetLayerColor(COLORREF clr){m_clrLayer = clr;}
	void SetLayerVisibility(BOOL bVisible){ m_bLayerVisible = bVisible;}
	void SetAsActiveLayer(){ m_bActiveLayer = TRUE;}
	void SetAsInactiveLayer(){ m_bActiveLayer = FALSE;}

	void Serialize(CArchive &ar);
};
#endif // __LAYER_H__
